@@ -23,6 +23,7 @@ from types import SimpleNamespace
 from typing import List
 
 import click
+import rasterio as rio
 import rasterio.crs as rio_crs
 from click.core import ParameterSource
 from geedim import schema, Initialize, version
@@ -56,15 +57,16 @@ class ChainedCommand(click.Command):
         """ Strip some RST markup from the help text for CLI display.  Assumes no grid tables. """
         if not hasattr(self, 'click_wrap_text'):
             self.click_wrap_text = click.formatting.wrap_text
-        # TODO: copy homonim sub_strings
         sub_strings = {
-            '\b\n': '\n\b',  # convert from RST friendly to click literal (unwrapped) block marker
-            ':option:': '',  # strip ':option:'
-            '\| ': '',  # strip RST literal (unwrapped) marker in e.g. tables and bullet lists
-            '\n\.\. _.*:\n': '',  # strip RST ref directive '\n.. <name>:\n'
-            '`(.*)<(.*)>`_': '\g<1>',  # convert from RST cross-ref '`<name> <<link>>`_' to 'name'
-            '::': ':'  # convert from RST '::' to ':'
-        }
+            '\b\n': '\n\b',                 # convert from RST friendly to click literal (unwrapped) block marker
+            r'\| ': '',                     # strip RST literal (unwrapped) marker in e.g. tables and bullet lists
+            '\n\.\. _.*:\n': '',            # strip RST ref directive '\n.. _<name>:\n'
+            '`(.*?) <(.*?)>`_': r'\g<1>',   # convert from RST cross-ref '`<name> <<link>>`_' to 'name'
+            '::': ':',                      # convert from RST '::' to ':'
+            '``(.*?)``': r'\g<1>',          # convert from RST '``literal``' to 'literal'
+            ':option:`(.*?)( <.*?>)?`': r'\g<1>',  # convert ':option:`--name <group-command --name>`' to '--name'
+            ':option:`(.*?)`': r'\g<1>',    # convert ':option:`--name`' to '--name'
+        }  # yapf: disable
 
         def reformat_text(text, width, **kwargs):
             for sub_key, sub_value in sub_strings.items():
@@ -91,6 +93,13 @@ class ChainedCommand(click.Command):
             # append any image id's to the image_list
             ctx.obj.image_list += list(ctx.params['image_id'])
 
+        if ('like' in ctx.params) and (ctx.params['like'] is not None):
+            # populate crs, crs_transform & shape parameters from a template raster
+            with rio.open(ctx.params['like'], 'r') as im:
+                ctx.params['crs'] = f'EPSG:{im.crs.to_epsg()}'  # TODO: WKT?
+                ctx.params['crs_transform'] = im.transform
+                ctx.params['shape'] = im.shape
+
         return click.Command.invoke(self, ctx)
 
 
@@ -189,15 +198,14 @@ def _prepare_image_list(obj: SimpleNamespace, mask=False) -> List[MaskedImage, ]
             raise ValueError(f'Unsupported image object type: {type(im_obj)}')
         image_list.append(im_obj)
 
-    if obj.region is None and any([not im.has_fixed_projection for im in image_list]):
-        raise click.BadOptionUsage('region', 'One of --region or --bbox is required for a composite image.')
     return image_list
 
 
 # Define click options that are common to more than one command
 bbox_option = click.option(
     '-b', '--bbox', type=click.FLOAT, nargs=4, default=None, callback=_bbox_cb,
-    help='Region defined by WGS84 bounding box co-ordinates (left, bottom, right, top).'
+    metavar='LEFT BOTTOM RIGHT TOP',
+    help='Region defined by WGS84 bounding box co-ordinates.'
 )
 region_option = click.option(
     '-r', '--region', type=click.Path(exists=True, dir_okay=False, allow_dash=True), default=None, callback=_region_cb,
@@ -230,6 +238,20 @@ scale_offset_option = click.option(
     '-so/-nso', '--scale-offset/--no-scale-offset', default=False, show_default=True,
     help='Whether to apply any EE band scales and offsets to the image.'
 )
+crs_transform_option = click.option(
+    '-ct', '--crs-transform', type=click.FLOAT, nargs=6, default=None,
+    metavar='XSCALE XSHEAR XTRANSLATION YSHEAR YSCALE YTRANSLATION',
+    help='Six element affine transform in the download CRS.  Use with ``--shape`` to specify image '
+         'bounds and resolution.'
+)  # yapf: disable
+shape_option = click.option(
+    '-sh', '--shape', type=click.INT, nargs=2, default=None, metavar='HEIGHT WIDTH',
+    help='Image height & width dimensions (pixels).'
+)
+like_option = click.option(
+    '-l', '--like', type=click.Path(exists=True, dir_okay=False), default=None,
+    help='Template raster from which to derive ``--crs``, ``--crs-transform`` & ``--shape``.'
+)
 
 
 # geedim CLI and chained command group
@@ -246,14 +268,17 @@ def cli(ctx, verbose, quiet):
 
 
 # TODO: add clear docs on what is piped out of or into each command.
-# TODO: use cloup and linear help layout.  move lists of option values from command docting to corresponding option help
+# TODO: use cloup and linear help layout & move lists of option values from command docsting to corresponding option
+#  help
+# TODO: add RST option markup like in homonim e.g. :option:`--mask-method`, and
+#  :option:`--param-image <homonim-fuse --param-image>`
 
 # config command
 @click.command(cls=ChainedCommand, context_settings=dict(auto_envvar_prefix='GEEDIM'))
 @click.option(
     '-mc/-nmc', '--mask-cirrus/--no-mask-cirrus', default=True, show_default=True,
     help='Whether to mask cirrus clouds.  Valid for Landsat 8-9 images, and, for Sentinel-2 images with '
-    'the `qa` :option:`--mask-method`.'
+    'the `qa` ``--mask-method``.'
 )
 @click.option(
     '-ms/-nms', '--mask-shadows/--no-mask-shadows', default=True, show_default=True,
@@ -266,7 +291,7 @@ def cli(ctx, verbose, quiet):
 )
 @click.option(
     '-p', '--prob', type=click.FloatRange(min=0, max=100), default=60, show_default=True,
-    help='Cloud probability threshold (%). Valid for Sentinel-2 images with the `cloud-prob` :option:`--mask-method`'
+    help='Cloud probability threshold (%). Valid for Sentinel-2 images with the `cloud-prob` ``--mask-method``'
 )
 @click.option(
     '-d', '--dark', type=click.FloatRange(min=0, max=1), default=.15, show_default=True,
@@ -371,7 +396,7 @@ cli.add_command(config)
 @click.option(
     '-cp', '--cloudless-portion', type=click.FloatRange(min=0, max=100), default=0, show_default=True,
     help='Lower limit on the cloud/shadow free portion of the region (%).  If cloud/shadow masking is not supported '
-    'for the specified collection, :option:`--cloudless-portion` will operate like :option:`--fill-portion`.'
+    'for the specified collection, ``--cloudless-portion`` will operate like ``--fill-portion``.'
 )
 @click.option(
     '-cf', '--custom-filter', type=click.STRING, default=None,
@@ -474,18 +499,21 @@ cli.add_command(search)
 # download command
 @click.command(cls=ChainedCommand)
 @click.option('-i', '--id', 'image_id', type=click.STRING, multiple=True, help='Earth Engine image ID(s) to download.')
+@crs_option
 @bbox_option
 @region_option
-@click.option(
-    '-dd', '--download-dir', type=click.Path(exists=True, file_okay=False, dir_okay=True, writable=True), default=None,
-    show_default='current working directory.', help='Directory to download image file(s) into.'
-)
-@crs_option
 @scale_option
+@crs_transform_option
+@shape_option
+@like_option
 @dtype_option
 @mask_option
 @resampling_option
 @scale_offset_option
+@click.option(
+    '-dd', '--download-dir', type=click.Path(exists=True, file_okay=False, dir_okay=True, writable=True), default=None,
+    show_default='current working directory.', help='Directory to download image file(s) into.'
+)
 @click.option(
     '-mts', '--max-tile-size', type=click.FLOAT, default=BaseImage._ee_max_tile_size, show_default=True,
     help='Maximum download tile size (MB).'
@@ -496,16 +524,16 @@ cli.add_command(search)
 )
 @click.option('-o', '--overwrite', is_flag=True, default=False, help='Overwrite the destination file if it exists.')
 @click.pass_obj
-def download(obj, image_id, bbox, region, download_dir, mask, max_tile_size, max_tile_dim, overwrite, **kwargs):
+def download(obj, image_id, bbox, region, like, download_dir, mask, max_tile_size, max_tile_dim, overwrite, **kwargs):
     # @formatter:off
     """
     Download image(s).
 
     Download Earth Engine image(s) to GeoTIFF file(s), allowing optional region of interest, and other image
-    formatting options to be specified.  Images larger than the `Earth Engine size limit
-    <https://developers.google.com/earth-engine/apidocs/ee-image-getdownloadurl>`_ are split and downloaded as separate
-    tiles, then re-assembled into a single GeoTIFF.  Downloaded image files are populated with metadata from the Earth
-    Engine image and STAC.
+    formatting options to be specified.  Images larger than the
+    `Earth Engine size limit <https://developers.google.com/earth-engine/apidocs/ee-image-getdownloadurl>`_ are split
+    and downloaded as separate tiles, then re-assembled into a single GeoTIFF.  Downloaded image files are populated
+    with metadata from the Earth Engine image and STAC.
 
     This command can be chained after the ``composite`` command, to download the composite image.  It can also be
     chained after the ``search`` command, in which case the search result images will be downloaded, without the need
@@ -526,8 +554,13 @@ def download(obj, image_id, bbox, region, download_dir, mask, max_tile_size, max
 
     Images from other collections, will contain the FILL_MASK band only.
 
-    If neither ``--bbox`` or ``--region`` are specified, the entire image granule
-    will be downloaded.
+    Bounds and resolution of the downloaded image can be specified with ``--region`` / ``--bbox`` and ``--scale`` /
+    ``--shape``, or ``--crs-transform`` and ``--shape``.  The ``--like`` option will automatically derive ``--crs``,
+    ``--crs-transform`` and ``--shape`` from a provided template raster.  If no bounds are specified (with either
+    ``--region``, or ``--crs-transform`` & ``--shape``), the entire image granule is downloaded.
+
+    When ``--crs``, ``--scale``, ``--crs-transform`` and ``--shape`` are not specified, the pixel grids of the
+    downloaded and Earth Engine images will coincide.
 
     Image filenames are derived from their Earth Engine ID.
     \b
@@ -561,23 +594,26 @@ cli.add_command(download)
 # export command
 @click.command(cls=ChainedCommand)
 @click.option('-i', '--id', 'image_id', type=click.STRING, multiple=True, help='Earth Engine image ID(s) to export.')
+@crs_option
 @bbox_option
 @region_option
-@click.option(
-    '-df', '--drive-folder', type=click.STRING, default=None, show_default='root folder.',
-    help='Google Drive folder to export image(s) to.'
-)
-@crs_option
 @scale_option
+@crs_transform_option
+@shape_option
+@like_option
 @dtype_option
 @mask_option
 @resampling_option
 @scale_offset_option
+@click.option(
+    '-df', '--drive-folder', type=click.STRING, default=None, show_default='root folder.',
+    help='Google Drive folder to export image(s) to.'
+)
 @click.option(
     '-w/-nw', '--wait/--no-wait', default=True, show_default=True, help='Whether to wait for the export to complete.'
 )
 @click.pass_obj
-def export(obj, image_id, bbox, region, drive_folder, mask, wait, **kwargs):
+def export(obj, image_id, bbox, region, like, drive_folder, mask, wait, **kwargs):
     # @formatter:off
     """
     Export image(s) to Google Drive.
@@ -604,8 +640,13 @@ def export(obj, image_id, bbox, region, drive_folder, mask, wait, **kwargs):
 
     Images from other collections, will contain the FILL_MASK band only.
 
-    If neither ``--bbox`` or ``--region`` are specified, the entire image granule
-    will be downloaded.
+    Bounds and resolution of the exported image can be specified with ``--region`` / ``--bbox`` and ``--scale`` /
+    ``--shape``, or ``--crs-transform`` and ``--shape``.  The ``--like`` option will automatically derive ``--crs``,
+    ``--crs-transform`` and ``--shape`` from a provided template raster.  If no bounds are specified (with either
+    ``--region``, or ``--crs-transform`` & ``--shape``), the entire image granule is exported.
+
+    When ``--crs``, ``--scale``, ``--crs-transform`` and ``--shape`` are not specified, the pixel grids of the
+    exported and Earth Engine images will coincide.
 
     Image filenames are derived from their Earth Engine ID.
     \b
@@ -662,17 +703,17 @@ cli.add_command(export)
 @click.option(
     '-b', '--bbox', type=click.FLOAT, nargs=4, default=None, callback=_bbox_cb,
     help='Give preference to images with the highest cloudless (or filled) portion inside this bounding box (left, '
-    'bottom, right, top).  Valid for `mosaic` and `q-mosaic` compositing :option:`--method`.'
+    'bottom, right, top).  Valid for `mosaic` and `q-mosaic` compositing ``--method``.'
 )
 @click.option(
     '-r', '--region', type=click.Path(exists=True, dir_okay=False, allow_dash=True), default=None, callback=_region_cb,
     help='Give preference to images with the highest cloudless (or filled) portion inside this geojson polygon, '
-    'or raster file, region.  Valid for `mosaic` and `q-mosaic` compositing :option:`--method`.'
+    'or raster file, region.  Valid for `mosaic` and `q-mosaic` compositing ``--method``.'
 )
 @click.option(
     '-d', '--date', type=click.DateTime(),
     help='Give preference to images closest to this date (UTC).  Valid for `mosaic` and `q-mosaic` compositing '
-    ':option:`--method`.'
+    '``--method``.'
 )
 @click.pass_obj
 def composite(obj, image_id, mask, method, resampling, bbox, region, date):
